@@ -2,7 +2,34 @@
 
 import copy
 import inspect
+import pprint
 import re
+import sys
+import os
+import pdb
+import cProfile
+import pstats
+import urlparse
+import cgi
+from wsgiref.simple_server import make_server
+from wsgiref.handlers import SimpleHandler
+
+SIMPLEAPI_DEBUG = bool(int(os.environ.get('SIMPLEAPI_DEBUG', 0)))
+SIMPLEAPI_DEBUG_FILENAME = os.environ.get('SIMPLEAPI_DEBUG_FILENAME',
+    'simpleapi.profile')
+SIMPLEAPI_DEBUG_LEVEL = os.environ.get('SIMPLEAPI_DEBUG_LEVEL', 'all')
+assert SIMPLEAPI_DEBUG_LEVEL in ['all', 'call'], \
+    u'SIMPLEAPI_DEBUG_LEVEL must be one of these: all, call'
+
+TRIGGERED_METHODS = ['get', 'post', 'put', 'delete']
+FRAMEWORKS = ['flask', 'django', 'appengine', 'dummy', 'standalone', 'wsgi']
+MAX_CONTENT_LENGTH = 1024 * 1024 * 16 # 16 megabytes
+
+try:
+    from google.appengine.ext.webapp import RequestHandler as AE_RequestHandler
+    has_appengine = True
+except ImportError:
+    has_appengine = False
 
 from sapirequest import SAPIRequest
 from request import Request, RequestException
@@ -11,26 +38,184 @@ from namespace import NamespaceException
 from feature import __features__, Feature, FeatureException
 from simpleapi.message import formatters, wrappers
 from utils import glob_list
+import logging
 
 __all__ = ('Route', )
 
-class RouteException(Exception): pass
 class Route(object):
 
+    def __new__(cls, *args, **kwargs):
+        if kwargs.get('framework') == 'appengine':
+            assert has_appengine
+            class AppEngineRouter(AE_RequestHandler):
+                def __getattribute__(self, name):
+                    if name in TRIGGERED_METHODS:
+                        self.request.method = name
+                        return self
+                    else:
+                        return AE_RequestHandler.__getattribute__(self, name)
+                                
+                def __call__(self):
+                    result = self.router(self.request)
+                    self.response.out.write(result['result'])
+            
+            AppEngineRouter.router = Router(*args, **kwargs)
+            return AppEngineRouter
+        elif kwargs.get('framework') == 'flask':
+            obj = Router(*args, **kwargs)
+            obj.__name__ = 'Route'
+            return obj
+        elif kwargs.get('framework') == 'wsgi':
+            router = Router(*args, **kwargs)
+            class WSGIHandler(object):
+                def __call__(self, *args, **kwargs):
+                    return self.router.handle_request(*args, **kwargs)
+            handler = WSGIHandler()
+            handler.router = router
+            return handler
+        else:
+            return Router(*args, **kwargs)
+
+class StandaloneRequest(object): pass
+class RouterException(Exception): pass
+class Router(object):
+
     def __init__(self, *namespaces, **kwargs):
         """Takes at least one namespace. 
         """
         self.nmap = {}
+        self.debug = kwargs.get('debug', False)
         self.restful = kwargs.get('restful', False)
         self.framework = kwargs.get('framework', 'django')
-        assert self.framework in ['flask', 'django']
+        self.path = re.compile(kwargs.get('path', r'^/'))
         
-        # for Flask support:
-        self.__name__ = 'Route'
+        # make shortcut 
+        self._caller = self.__call__
+        
+        assert self.framework in FRAMEWORKS
+        assert (self.debug ^ SIMPLEAPI_DEBUG) or \
+            not (self.debug and SIMPLEAPI_DEBUG), \
+            u'You can either activate Route-debug or simpleapi-debug, not both.'
+
+        if self.debug or SIMPLEAPI_DEBUG:
+            logging.basicConfig(level=logging.DEBUG)
+        else:
+            logging.basicConfig(level=logging.WARNING)
+        
+        if SIMPLEAPI_DEBUG and SIMPLEAPI_DEBUG_LEVEL == 'all':
+            self.profile_start()
 
         for namespace in namespaces:
             self.add_namespace(namespace)
 
+    def handle_request(self, environ, start_response):
+        if not self.path.match(environ.get('PATH_INFO')): 
+            status = '404 Not found'
+            start_response(status, [])
+            return ["Entry point not found"]
+        else:
+            content_type = environ.get('CONTENT_TYPE')
+            try:
+                content_length = int(environ['CONTENT_LENGTH'])
+            except (KeyError, ValueError):
+                content_length = 0
+
+            # make sure we ignore too large requests for security and stability
+            # reasons
+            if content_length > MAX_CONTENT_LENGTH:
+                status = '413 Request entity too large'
+                start_response(status, [])
+                return ["Request entity too large"]
+
+            request_method = environ.get('REQUEST_METHOD', '').lower()
+
+            # make sure we only support methods we care
+            if not request_method in TRIGGERED_METHODS:
+                status = '501 Not Implemented'
+                start_response(status, [])
+                return ["Not Implemented"]
+
+            query_get = urlparse.parse_qs(environ.get('QUERY_STRING'))
+            for key, value in query_get.iteritems():
+                query_get[key] = value[0] # respect the first value only
+
+            query_post = {}
+            if content_type in ['application/x-www-form-urlencoded', 
+                'application/x-url-encoded']:
+                post_env = environ.copy()
+                post_env['QUERY_STRING'] = ''
+                fs = cgi.FieldStorage(
+                    fp=environ['wsgi.input'],
+                    environ=post_env,
+                    keep_blank_values=True
+                )
+                query_post = {}
+                for key in fs:
+                    query_post[key] = fs.getvalue(key)
+            elif content_type == 'multipart/form-data':
+                # XXX TODO
+                raise NotImplementedError, u'Currently not supported.' 
+            
+            # GET + POST 
+            query_data = query_get
+            query_data.update(query_post)
+
+            # Make request
+            request = StandaloneRequest()
+            request.method = request_method
+            request.data = query_data
+            request.remote_addr = environ.get('REMOTE_ADDR', '')
+
+            # Make call
+            result = self._caller(request)
+
+            status = '200 OK'
+            headers = [('Content-type', result['mimetype'])]
+            start_response(status, headers)
+            return [result['result'],]
+
+    def serve(self, host='', port=5050):
+        httpd = make_server(host, port, self.handle_request)
+        logging.info(u"Started serving on port %d..." % port)
+        try:
+            httpd.serve_forever()
+        except KeyboardInterrupt:
+            logging.info(u"Server stopped.")
+
+    def profile_start(self):
+        self.profile = cProfile.Profile()
+        self.profile.enable()
+    
+    def profile_stop(self):
+        self.profile.disable()
+        self.profile.dump_stats(SIMPLEAPI_DEBUG_FILENAME)
+
+    def profile_stats(self):
+        logging.debug(u"Loading stats...")
+        stats = pstats.Stats(SIMPLEAPI_DEBUG_FILENAME)
+        stats.strip_dirs().sort_stats('time', 'calls') \
+            .print_stats()
+
+    def __del__(self):
+        if SIMPLEAPI_DEBUG and SIMPLEAPI_DEBUG_LEVEL == 'all':
+            self.profile_stop()
+            self.profile_stats()
+
+    def is_standalone(self):
+        return self.framework in ['standalone', 'wsgi']
+
+    def is_dummy(self):
+        return self.framework == 'dummy'
+
+    def is_appengine(self):
+        return self.framework == 'appengine'
+
+    def is_flask(self):
+        return self.framework == 'flask'
+
+    def is_django(self):
+        return self.framework == 'django'
+
     def _redefine_default_namespace(self):
         # - recalculate default namespace version -
         # if map has no default version, determine namespace with the
@@ -49,7 +234,8 @@ class Route(object):
 
     def add_namespace(self, namespace):
         version = getattr(namespace, '__version__', 1)
-        assert isinstance(version, int), u'version must be either an integer or not set'
+        assert isinstance(version, int), \
+            u'version must be either an integer or not set'
 
         # make sure no version is assigned twice
         assert not self.nmap.has_key(version), u'version is assigned twice'
@@ -153,8 +339,11 @@ class Route(object):
         if hasattr(namespace, '__authentication__'):
             authentication = namespace.__authentication__
             if isinstance(authentication, basestring):
-                authentication = lambda namespace, access_key: \
-                    namespace.__authentication__ == access_key
+                if hasattr(namespace, authentication):
+                    authentication = getattr(namespace, authentication)
+                else:
+                    authentication = lambda namespace, access_key: \
+                        namespace.__authentication__ == access_key
         else:
             # grant allow everyone access
             authentication = lambda namespace, access_key: True
@@ -166,10 +355,12 @@ class Route(object):
 
             if isinstance(ip_restriction, list):
                 # make the ip address list wildcard searchable
-                namespace.__ip_restriction__ = glob_list(namespace.__ip_restriction__)
+                namespace.__ip_restriction__ = \
+                    glob_list(namespace.__ip_restriction__)
 
                 # restrict access to the given ip address list
-                ip_restriction = lambda namespace, ip: ip in namespace.__ip_restriction__
+                ip_restriction = lambda namespace, ip: ip in \
+                    namespace.__ip_restriction__
         else:
             # accept every ip address
             ip_restriction = lambda namespace, ip: True
@@ -213,11 +404,13 @@ class Route(object):
         if hasattr(namespace, '__features__'):
             raw_features = namespace.__features__
             for feature in raw_features:
-                assert isinstance(feature, basestring) or issubclass(feature, Feature)
+                assert isinstance(feature, basestring) or \
+                    issubclass(feature, Feature)
+
                 if isinstance(feature, basestring):
                     assert feature in __features__.keys(), \
                         u'%s is not a built-in feature' % feature
-                    
+
                     features.append(__features__[feature](self.nmap[version]))
                 elif issubclass(feature, Feature):
                     features.append(feature(self.nmap[version]))
@@ -232,7 +425,11 @@ class Route(object):
 
         request_items = dict(sapi_request.REQUEST.items())
         request_items.update(urlparameters)
-        
+
+        if SIMPLEAPI_DEBUG and SIMPLEAPI_DEBUG_LEVEL == 'call':
+            logging.info(pprint.pformat(request_items))
+            self.profile_start()
+
         version = request_items.pop('_version', 'default')
         callback = request_items.pop('_callback', None)
         output_formatter = request_items.pop('_output', None)
@@ -289,7 +486,7 @@ class Route(object):
 
             # check whether version exists or not
             if not self.nmap.has_key(version):
-                raise RouteException(u'Version %s not found (possible: %s)' % \
+                raise RouterException(u'Version %s not found (possible: %s)' % \
                     (version, ", ".join(map(lambda i: str(i), self.nmap.keys()))))
 
             request = Request(
@@ -300,13 +497,14 @@ class Route(object):
                 wrapper=wrapper_instance,
                 callback=callback,
                 mimetype=mimetype,
-                restful=self.restful
+                restful=self.restful,
+                debug=self.debug
             )
             response = request.run(request_items)
             http_response = response.build()
         except Exception, e:
             if isinstance(e, (NamespaceException, RequestException,
-               ResponseException, RouteException, FeatureException)):
+               ResponseException, RouterException, FeatureException)):
                 err_msg = unicode(e)
             else:
                 err_msg = u'An internal error occurred during your request.'
@@ -319,15 +517,19 @@ class Route(object):
                 msgs.append('')
                 msgs.append(u'------- Traceback follows -------')
                 for idx, item in enumerate(trace):
-                    msgs.append(u"(%s)\t%s:%s (%s)" % (idx+1, item[3], item[2], item[1]))
+                    msgs.append(u"(%s)\t%s:%s (%s)" % 
+                        (idx+1, item[3], item[2], item[1]))
                     if item[4]:
                         for line in item[4]:
                             msgs.append(u"\t\t%s" % line.strip())
                     msgs.append('') # blank line
                 msgs.append('     -- End of traceback --     ')
                 msgs.append('')
+                logging.error("\n".join(msgs))
 
-                print "\n".join(msgs) # TODO: send it to the admins by email!
+                if self.debug:
+                    e, m, tb = sys.exc_info()
+                    pdb.post_mortem(tb)
 
             response = Response(
                 sapi_request,
@@ -338,4 +540,8 @@ class Route(object):
             )
             http_response = response.build(skip_features=True)
 
+        if SIMPLEAPI_DEBUG and SIMPLEAPI_DEBUG_LEVEL == 'call':
+            self.profile_stop()
+            self.profile_stats()
+
         return http_response
\ No newline at end of file
